@@ -1,27 +1,29 @@
-# Currently this function could only parse svg file created by
-# CairoSVG() in Cairo package
+# Currently this function could only parse svg files created by
+# the cairo graphics library, typically from svg() in the grDevices
+# package (R >= 2.14.0 required for Windows OS), and CairoSVG() in the
+# Cairo package.
 parseSVG = function(file.name)
 {
-	# Use XML package
+    # Use XML package
     svgFile = xmlParse(file.name);
-	# Don't forget the name space!
+    # Don't forget the name space!
     newXMLNamespace(xmlRoot(svgFile), "http://www.w3.org/2000/svg", "svg");
-	# Find the first <g> child of <svg>
+    # Find the first <g> child of <svg>
     pathRoot = getNodeSet(svgFile, "/svg:svg/svg:g");
-	if(!length(pathRoot)) stop(sprintf("Failed in parsing file '%s'",
-									   file.name));
-	pathRoot = pathRoot[[1]];
+    if(!length(pathRoot)) stop(sprintf("Failed in parsing file '%s'",
+                                       file.name));
+    pathRoot = pathRoot[[1]];
     
-	# Default style for a <path> node
+    # Default style for a <path> node
     defaultStyle = c("stroke" = "none",
-                    "stroke-width" = "1",
-                    "stroke-linecap" = "butt",
-                    "stroke-linejoin" = "miter",
-                    "stroke-miterlimit" = "4",
-                    "stroke-opacity" = "1",
-                    "fill" = "rgb(0%,0%,0%)",
-                    "fill-rule" = "nonzero",
-                    "fill-opacity" = "1");
+                     "stroke-width" = "1",
+                     "stroke-linecap" = "butt",
+                     "stroke-linejoin" = "miter",
+                     "stroke-miterlimit" = "4",
+                     "stroke-opacity" = "1",
+                     "fill" = "rgb(0%,0%,0%)",
+                     "fill-rule" = "nonzero",
+                     "fill-opacity" = "1");
     
     # Handle <path> style in named vector
     parseStyle = function(style)
@@ -62,13 +64,13 @@ parseSVG = function(file.name)
         return(style);
     }
     # Parse <path> and <use> nodes into structured lists
-	#
-	# <path style="" d="">   =====>   style=..., d=..., x=0, y=0
-	#
-	# <use xlink:href="#glyph0-0" x="63.046875" y="385.921875"/>
-	# =====>
-	# style=..., d=..., x=63.046875, y=385.921875
-	#
+    #
+    # <path style="" d="">   =====>   style=..., d=..., x=0, y=0
+    #
+    # <use xlink:href="#glyph0-0" x="63.046875" y="385.921875"/>
+    # =====>
+    # style=..., d=..., x=63.046875, y=385.921875
+    #
     parseNode = function(node)
     {
         if(xmlName(node) == "use")
@@ -85,24 +87,24 @@ parseSVG = function(file.name)
             style = updateStyleUpward(node);
             d = xmlAttrs(node)["d"];
             x = y = 0;
-        } else stop("Unknown child node of '/svg/g'");
+        } else return(NULL);
         xy = as.numeric(c(x, y));
         names(d) = NULL;
         names(xy) = NULL;
         return(list(style = style, d = d, xy = xy));
     }
     # Flatten nodes
-	# <g>
-	#   <use />
-	#   <use />
-	#   <use />
-	# </g>
-	#
-	# =====>
-	#
-	# <use />
-	# <use />
-	# <use />
+    # <g>
+    #   <use />
+    #   <use />
+    #   <use />
+    # </g>
+    #
+    # =====>
+    #
+    # <use />
+    # <use />
+    # <use />
     expandNode = function(node)
     {
         children = xmlChildren(node);
@@ -112,6 +114,9 @@ parseSVG = function(file.name)
     nodes = unlist(xmlSApply(pathRoot, expandNode));
     names(nodes) = NULL;
     paths = lapply(nodes, parseNode);
+    path.is.null = sapply(paths, is.null);
+    paths[path.is.null] = NULL;
+    if(!length(paths)) stop("Unknown child node of '/svg/g'");
     return(paths);
 }
 
@@ -123,58 +128,59 @@ parseSVG = function(file.name)
 ##'
 ##' This function uses the XML package in R and a subset of librsvg
 ##' (\url{http://librsvg.sourceforge.net/}) to parse the SVG file, and
-##' utilize the Ming library (\url{http://www.libming.org/}) to
-##' implement the conversion. Currently the support for SVG file is
-##' \strong{NOT} perfect and \strong{ONLY} support files generated by
-##' \code{\link[Cairo]{CairoSVG}()} in the \pkg{Cairo} package.
+##' uses the Ming library (\url{http://www.libming.org/}) to
+##' implement the conversion. Currently this function supports SVG files
+##' created by \code{\link[grDevices]{svg}()} in the \pkg{grDevices}
+##' package, and \code{\link[Cairo]{CairoSVG}()} in the
+##' \pkg{Cairo} package.
 ##'
 ##' @param input the file names of the SVG files to be converted
 ##' @param output the name of the output SWF file
 ##' @param bgColor background color of the output SWF file
 ##' @param interval the time interval (in seconds) between animation frames
 ##' @return The path of the generated SWF file if successful.
-##' @note Currently there is no function in R that could auto-generate
-##' a sequence of SVG files, so you may create the them once at a time.
-##' See the example.
 ##' @export
 ##' @author Yixuan Qiu <\email{yixuan.qiu@@cos.name}>
-##' @examples \dontrun{if(require("Cairo")) {
+##' @examples \dontrun{if(capabilities("cairo")) {
 ##'   olddir = setwd(tempdir())
-##'   filenames = sprintf("Rplot%03d.svg", 1:9)
-##'   for(fn in filenames) {
-##'     CairoSVG(fn)
-##'     plot(runif(20), ylim = c(0, 1))
-##'     dev.off()
+##'   svg("Rplot%03d.svg", onefile = FALSE)
+##'   set.seed(123)
+##'   x = rnorm(5)
+##'   y = rnorm(5)
+##'   for(i in 1:100) {
+##'       plot(x <- x + 0.1 * rnorm(5), y <- y + 0.1 * rnorm(5),
+##'            xlim = c(-3, 3), ylim = c(-3, 3), col = "steelblue",
+##'            pch = 16, cex = 2, xlab = "x", ylab = "y")
 ##'   }
-##'   output = svg2swf(filenames, bgColor = "steelblue")
+##'   dev.off()
+##'   output = svg2swf(sprintf("Rplot%03d.svg", 1:100), interval = 0.1)
 ##'   swf2html(output)
 ##'   setwd(olddir)
 ##' }
 ##' }
 ##'
 svg2swf = function(input, output = "./movie.swf", bgColor = "white",
-				   interval = 1)
+                   interval = 1)
 {
     if(!inherits(input, "character")) 
         stop("'input' must be a character vector naming the input SVG files");
-	
-	outDir = dirname(output);
-	outFile = basename(output);
-	bg = col2rgb(bgColor, alpha = FALSE);
-	bg = as.integer(bg);
-
-	filesData = lapply(input, parseSVG);
+    
+    outDir = dirname(output);
+    outFile = basename(output);
+    bg = col2rgb(bgColor, alpha = FALSE);
+    bg = as.integer(bg);
+    
+    filesData = lapply(input, parseSVG);
     firstFile = xmlParse(input[1]);
     size = xmlAttrs(xmlRoot(firstFile))["viewBox"];
     size = as.numeric(unlist(strsplit(size, " ")));
-
-	oldwd = setwd(outDir);
+    
+    oldwd = setwd(outDir);
     .Call("svg2swf", filesData, as.character(outFile), size,
-		  bg, as.numeric(interval), PACKAGE = "R2SWF");
+          bg, as.numeric(interval), PACKAGE = "R2SWF");
     setwd(oldwd);
-
-	output = normalizePath(output);
-	cat("SWF file created at ", output, ".\n", sep = "");
-	invisible(output);
+    
+    output = normalizePath(output);
+    cat("SWF file created at ", output, ".\n", sep = "");
+    invisible(output);
 }
-
